@@ -3,14 +3,27 @@
 #@+node:ekr.20150514035943.1: * @file ../commands/baseCommands.py
 #@@first
 """The base class for all of Leo's user commands."""
+from typing import Any, Tuple, TYPE_CHECKING
 from leo.core import leoGlobals as g
+#@+<< baseCommands annotations >>
+#@+node:ekr.20220828071357.1: ** << baseCommands annotations >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.plugins.qt_text import QTextEditWrapper as Wrapper
+else:
+    Cmdr = Any
+    Event = Any
+    Wrapper = Any
+Widget = Any
+#@-<< baseCommands annotations >>
 #@+others
 #@+node:ekr.20160514095639.1: ** class BaseEditCommandsClass
 class BaseEditCommandsClass:
     """The base class for all edit command classes"""
     #@+others
     #@+node:ekr.20150516040334.1: *3* BaseEdit.ctor
-    def __init__(self, c):
+    def __init__(self, c: Cmdr) -> None:
         """
         Ctor for the BaseEditCommandsClass class.
 
@@ -20,7 +33,7 @@ class BaseEditCommandsClass:
         self.c = c
     #@+node:ekr.20150514043714.3: *3* BaseEdit.begin/endCommand (handles undo)
     #@+node:ekr.20150514043714.4: *4* BaseEdit.beginCommand
-    def beginCommand(self, w, undoType='Typing'):
+    def beginCommand(self, w: Wrapper, undoType: str='Typing') -> Wrapper:
         """Do the common processing at the start of each command."""
         c, p, u = self.c, self.c.p, self.c.undoer
         name = c.widget_name(w)
@@ -38,7 +51,7 @@ class BaseEditCommandsClass:
             self.undoData = None  # pragma: no cover
         return w
     #@+node:ekr.20150514043714.6: *4* BaseEdit.endCommand
-    def endCommand(self, label=None, changed=True, setLabel=True):
+    def endCommand(self, label: str=None, changed: bool=True, setLabel: bool=True) -> None:
         """
         Do the common processing at the end of each command.
         Handles undo only if we are in the body pane.
@@ -65,7 +78,7 @@ class BaseEditCommandsClass:
             else:
                 k.resetLabel()
     #@+node:ekr.20150514043714.7: *3* BaseEdit.editWidget
-    def editWidget(self, event, forceFocus=True):
+    def editWidget(self, event: Event, forceFocus: bool=True) -> Widget:
         """Return the edit widget for the event. Also sets self.w"""
         c = self.c
         w = event and event.widget
@@ -79,26 +92,26 @@ class BaseEditCommandsClass:
         self.w = w
         return w
     #@+node:ekr.20150514043714.8: *3* BaseEdit.getWSString
-    def getWSString(self, s):  # pragma: no cover
+    def getWSString(self, s: str) -> str:  # pragma: no cover
         """Return s with all characters replaced by tab or space."""
         return ''.join([ch if ch == '\t' else ' ' for ch in s])
     #@+node:ekr.20150514043714.9: *3* BaseEdit.oops
-    def oops(self):  # pragma: no cover
+    def oops(self) -> None:  # pragma: no cover
         """Return a "must be overridden" message"""
         g.pr("BaseEditCommandsClass oops:",
             g.callers(),
             "must be overridden in subclass")
     #@+node:ekr.20150514043714.10: *3* BaseEdit.Helpers
     #@+node:ekr.20150514043714.11: *4* BaseEdit._chckSel
-    def _chckSel(self, event, warning='no selection'):
+    def _chckSel(self, event: Event, warning: str='no selection') -> bool:
         """Return True if there is a selection in the edit widget."""
         w = self.editWidget(event)
-        val = w and w.hasSelection()
+        val = bool(w and w.hasSelection())
         if warning and not val:  # pragma: no cover
             g.es(warning, color='red')
         return val
     #@+node:ekr.20150514043714.13: *4* BaseEdit.getRectanglePoints
-    def getRectanglePoints(self, w):
+    def getRectanglePoints(self, w: Wrapper) -> Tuple[int, int, int, int]:
         """Return the rectangle corresponding to the selection range."""
         c = self.c
         c.widgetWantsFocusNow(w)
@@ -108,9 +121,9 @@ class BaseEditCommandsClass:
         r3, r4 = g.convertPythonIndexToRowCol(s, j)
         return r1 + 1, r2, r3 + 1, r4
     #@+node:ekr.20150514043714.14: *4* BaseEdit.keyboardQuit
-    def keyboardQuit(self, event=None):  # pragma: no cover
+    def keyboardQuit(self, event: Event=None) -> None:  # pragma: no cover
         """Clear the state and the minibuffer label."""
-        return self.c.k.keyboardQuit()
+        self.c.k.keyboardQuit()
     #@-others
 #@-others
 #@-leo
