@@ -20,15 +20,15 @@ from leo.core import leoAst
 #@-<< leoBeautify imports >>
 #@+<< leoBeautify annotations >>
 #@+node:ekr.20220822114959.1: ** << leoBeautify annotations >>
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
     from leo.core.leoNodes import Position, VNode
 else:
     Cmdr = Any
+    Event = Any
     Position = Any
     VNode = Any
-
-Event = Any
 #@-<< leoBeautify annotations >>
 #@+others
 #@+node:ekr.20191104201534.1: **   Top-level functions (leoBeautify.py)
