@@ -395,7 +395,6 @@ class ParserBaseClass:
             # get the patch fragment
             patch: List[Any] = []
             if p.hasChildren():
-                # self.doMenus(p.copy().firstChild(),kind,name,val,storeIn=patch)
                 self.doItems(p.copy(), patch)
             # setup
             parts = name.split()
@@ -404,9 +403,14 @@ class ParserBaseClass:
             targetPath, mode, source = parts
             if not targetPath.startswith('/'):
                 targetPath = '/' + targetPath
-            ans = self.patchMenuTree(g.app.config.menusList, targetPath)
+            is_local = 'leosettings' not in self.c.mFileName.lower()
+            if is_local:
+                mlist = g.app.config.menusList[:]
+                self.c.config.set(None, 'menus', 'menus', mlist)
+            else:
+                mlist = g.app.config.menusList
+            ans = self.patchMenuTree(mlist, targetPath)
             if ans:
-                # pylint: disable=unpacking-non-sequence
                 list_, idx = ans
                 if mode not in ('copy', 'cut'):
                     if source != 'clipboard':
@@ -440,6 +444,7 @@ class ParserBaseClass:
             pass  # #48: Not an error.
         else:
             g.es_print("ERROR: @menuat found but no menu tree to patch")
+
     #@+node:tbrown.20080514180046.9: *5* pbc.getName
     def getName(self, val: str, val2: str=None) -> str:
         if val2 and val2.strip():
@@ -484,7 +489,6 @@ class ParserBaseClass:
         return None
     #@+node:ekr.20070925144337.2: *4* pbc.doMenus & helper
     def doMenus(self, p: Position, kind: str, name: str, val: Any) -> None:
-
         c = self.c
         p = p.copy()
         aList: List[Any] = []  # This entire logic is mysterious, and likely buggy.
@@ -1780,8 +1784,22 @@ class LocalConfigManager:
     #@+node:ekr.20120215072959.12534: *5* c.config.getMenusList
     def getMenusList(self) -> List:
         """Return the list of entries for the @menus tree."""
+
+        # Typically empty, unless there is an @menuat setting.
         aList = self.get('menus', 'menus')
-        # aList is typically empty.
+
+        # Leo calls this method twice when loading an outline.
+        if not hasattr(self.c, 'menulist_pass'):
+            self.c.menulist_pass = 0
+        self.c.menulist_pass += 1
+
+        # Remove this outline's "doMenuat" settings so later outlines won't use them.
+        if self.c.menulist_pass == 2:
+            lm = g.app.loadManager
+            lm.globalSettingsDict['menus'] = None
+            self.set(None, 'menus', 'menus', None)
+            self.c.menulist_pass = 0
+
         return aList or g.app.config.menusList
     #@+node:ekr.20120215072959.12535: *5* c.config.getOpenWith
     def getOpenWith(self) -> List[Dict[str, Any]]:
@@ -1950,9 +1968,8 @@ class LocalConfigManager:
         if gs:
             assert isinstance(gs, g.GeneralSetting), repr(gs)
             path = gs.path
-            if warn and g.os_path_finalize(
-                c.mFileName) != g.os_path_finalize(path):  # #1341.
-                g.es("over-riding setting:", name, "from", path)
+            if warn and g.os_path_finalize(c.mFileName) != g.os_path_finalize(path):  # #1341.
+                g.trace("over-riding setting:", name, "from", path)
         d[key] = g.GeneralSetting(kind, path=c.mFileName, val=val, tag='setting')
     #@+node:ekr.20190905082644.1: *3* c.config.settingIsActiveInPath
     def settingIsActiveInPath(self, gs: str, target_path: str) -> bool:
