@@ -10,24 +10,29 @@ Leo's core calls this class to allocate all gui objects.
 
 Plugins may define their own gui classes by setting g.app.gui.
 """
-#@+<< imports leoGui.py >>
-#@+node:ekr.20220414080546.1: ** << imports leoGui.py >>
+#@+<< leoGui imports >>
+#@+node:ekr.20220414080546.1: ** << leoGui imports >>
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 from typing import TYPE_CHECKING
 from leo.core import leoGlobals as g
 from leo.core import leoFrame
-if TYPE_CHECKING:
+#@-<< leoGui imports >>
+#@+<< leoGui annotations >>
+#@+node:ekr.20220827062419.1: ** << leoGui annotations >>
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
-    from leo.core.leoNodes import Position as Pos
-    # from leo.core.leoQt import QtWidgets
-
+    from leo.core.leoGui import LeoKeyEvent as Event  # pylint: disable=import-self
+    from leo.core.leoNodes import Position
+    from leo.plugins.qt_frame import FindTabManager
+    from leo.plugins.qt_text import QTextEditWrapper as Wrapper
 else:
-    Cmdr = Pos = Any
-    # for NullGui and StringTextWrapper.
-Event = Any
+    Cmdr = Any
+    Event = Any
+    FindTabManager = Any
+    Position = Any
+    Wrapper = Any
 Widget = Any
-Wrapper = Any
-#@-<< imports leoGui.py >>
+#@-<< leoGui annotations >>
 #@+others
 #@+node:ekr.20031218072017.3720: ** class LeoGui
 class LeoGui:
@@ -41,16 +46,16 @@ class LeoGui:
         """Ctor for the LeoGui class."""
         self.active = False  # Used only by qt_gui.
         self.consoleOnly = True  # True if g.es goes to console.
-        self.globalFindTabManager: Any = None
+        self.globalFindTabManager: FindTabManager = None
         self.globalFindTab: Widget = None
-        self.idleTimeClass = None
+        self.idleTimeClass: Any = None  # Hard to annotate.
         self.isNullGui = False
-        self.lastFrame: Wrapper = None
+        self.lastFrame: Widget = None
         self.leoIcon = None
         self.mGuiName = guiName
         self.mainLoop = None
         self.plainTextWidget: Widget = None  # For SpellTabHandler class only.
-        self.root: Pos = None
+        self.root: Position = None
         self.script: Optional[str] = None
         self.splashScreen: Widget = None
         self.utils = None
@@ -99,7 +104,9 @@ class LeoGui:
     def destroySelf(self) -> None:
         self.oops()
     #@+node:ekr.20031218072017.3730: *4* LeoGui.dialogs
-    def runAboutLeoDialog(self, c: Cmdr, version: str, theCopyright: str, url: str, email: str) -> Any:
+    def runAboutLeoDialog(self,
+        c: Cmdr, version: str, theCopyright: str, url: str, email: str,
+    ) -> Any:  # Must be any, for compatibility with testing subclass.
         """Create and run Leo's About Leo dialog."""
         self.oops()
 
@@ -162,7 +169,7 @@ class LeoGui:
         title: str='Properties',
         data: str=None,
         callback: Callable=None,
-        buttons: str=None,
+        buttons: List[str]=None,
     ) -> Any:
         """Display a modal TkPropertiesDialog"""
         self.oops()
@@ -184,10 +191,6 @@ class LeoGui:
         self.oops()
         return 'no'
     #@+node:ekr.20031218072017.3732: *4* LeoGui.panels
-    def createColorPanel(self, c: Cmdr) -> None:
-        """Create a color panel"""
-        self.oops()
-
     def createComparePanel(self, c: Cmdr) -> None:
         """Create Compare panel."""
         self.oops()
@@ -257,7 +260,7 @@ class LeoGui:
         self,
         c: Cmdr,
         args: str=None,
-        p: Pos=None,
+        p: Position=None,
         script: str=None,
         buttonText: str=None,
         balloonText: str='Script Button',
@@ -295,7 +298,7 @@ class LeoGui:
     def put_help(self, c: Cmdr, s: str, short_title: str) -> None:
         pass
     #@+node:ekr.20051206103652: *4* LeoGui.widget_name (LeoGui)
-    def widget_name(self, w: Wrapper) -> str:
+    def widget_name(self, w: Widget) -> str:
         # First try the widget's getName method.
         if not 'w':
             return '<no widget>'
@@ -316,7 +319,7 @@ class LeoKeyEvent:
         char: str,
         event: Event,
         binding: Any,
-        w: Wrapper,
+        w: Any,
         x: int=None,
         y: int=None,
         x_root: int=None,
@@ -352,11 +355,11 @@ class LeoKeyEvent:
             d[ivar] = getattr(self, ivar)
         return f"LeoKeyEvent:\n{g.objToString(d)}"
     #@+node:ekr.20150511181702.1: *3* LeoKeyEvent.get & __getitem__
-    def get(self, attr: str) -> None:
+    def get(self, attr: str) -> Any:
         """Compatibility with g.bunch: return an attr."""
         return getattr(self, attr, None)
 
-    def __getitem__(self, attr: str) -> None:
+    def __getitem__(self, attr: str) -> Any:
         """Compatibility with g.bunch: return an attr."""
         return getattr(self, attr, None)
     #@+node:ekr.20140907103315.18775: *3* LeoKeyEvent.type
@@ -374,7 +377,7 @@ class NullGui(LeoGui):
         self.clipboardContents = ''
         self.focusWidget: Widget = None
         self.script = None
-        self.lastFrame: Wrapper = None  # The outer frame, to set g.app.log in runMainLoop.
+        self.lastFrame: Widget = None  # The outer frame, to set g.app.log in runMainLoop.
         self.isNullGui = True
         self.idleTimeClass: Any = g.NullObject
     #@+node:ekr.20031218072017.3744: *3* NullGui.dialogs
@@ -496,14 +499,11 @@ class NullGui(LeoGui):
 
     def onDeactivateEvent(self, *args: str, **keys: str) -> None:
         pass
-
-    def set_top_geometry(self, w: Wrapper, h: str, x: str, y: str) -> None:
-        pass
     #@+node:ekr.20070228155807: *3* NullGui.isTextWidget & isTextWrapper
-    def isTextWidget(self, w: Wrapper) -> bool:
+    def isTextWidget(self, w: Any) -> bool:
         return True  # Must be True for unit tests.
 
-    def isTextWrapper(self, w: Wrapper) -> bool:
+    def isTextWrapper(self, w: Any) -> bool:
         """Return True if w is a Text widget suitable for text-oriented commands."""
         return w and getattr(w, 'supportsHighLevelInterface', None)
     #@+node:ekr.20031218072017.2230: *3* NullGui.oops
