@@ -2,74 +2,103 @@
 #@+leo-ver=5-thin
 #@+node:ekr.20180121041003.1: * @file leoTips.py
 #@@first
-'''Save and show tips to the user.'''
+"""Save and show tips to the user."""
+#@+<< leoTips imports >>
+#@+node:ekr.20220901094023.1: ** << leoTips imports >>
 import random
-import leo.core.leoGlobals as g
+import textwrap
+from typing import Any, List, TYPE_CHECKING
+from leo.core import leoGlobals as g
+#@-<< leoTips imports >>
 assert g
+#@+<< leoTips annotatsions >>
+#@+node:ekr.20220901094043.1: ** << leoTips annotatsions >>
+if TYPE_CHECKING:  # pragma: no cover
+    from leo.core.leoCommands import Commands as Cmdr
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.core.leoNodes import Position
+else:
+    Cmdr = Any
+    Event = Any
+    Position = Any
+#@-<< leoTips annotatsions >>
+
+# Define constant strings for use in f-strings.
+at_s = "@"
+ref1_s = "<<"
+ref2_s = ">>"
+
 #@+others
 #@+node:ekr.20180121041252.1: ** class TipManager
-class TipManager(object):
-    '''A class to manage user tips.'''
-    
+#@@beautify
+
+
+class TipManager:
+    """A class to manage user tips."""
+
     key = 'shown-tips'
-    
     #@+others
     #@+node:ekr.20180121041748.1: *3* tipm.get_next_tip
-    def get_next_tip(self):
+    def get_next_tip(self) -> "UserTip":
         global tips
         db = g.app.db
         # Compute list of unseen tips.
         seen = db.get(self.key, [])
         unseen = [i for i in range(len(tips)) if i not in seen]
         if not unseen:
-            db [self.key] = []
+            db[self.key] = []
             unseen = list(range(len(tips)))
             seen = []
         # Choose a tip at random from the unseen tips.
         i = random.choice(unseen)
         assert i not in seen, (i, seen)
         seen.append(i)
-        db [self.key] = seen
+        db[self.key] = seen
         return tips[i]
     #@-others
 #@+node:ekr.20180121041301.1: ** class UserTip
-class UserTip(object):
-    '''A User Tip.'''
-    
-    def __init__(self, n=0, tags=None, text='', title=''):
-        self.n = n # Not used.
-        self.tags = tags or [] # Not used.
+#@@beautify
+
+
+class UserTip:
+    """A User Tip."""
+
+    def __init__(self, n: int=0, tags: List[str]=None, text: str='', title: str='') -> None:
+        self.n = n  # Not used.
+        self.tags: List[str] = tags or []  # Not used.
         self.title = title.strip()
         self.text = text.strip()
-        
-    def __repr__(self):
-        return '%s\n\n%s\n' % (self.title, self.text)
+
+    def __repr__(self) -> str:
+        return f"{self.title}\n\n{self.text}\n"
 
     __str__ = __repr__
 #@+node:ekr.20180121045646.1: ** make_tips (leoTips.py)
-def make_tips(c):
-    '''
+#@@beautify
+
+def make_tips(c: Cmdr) -> None:
+    """
     A script to make entries in the global tips array.
-    
+
     Each printed entry has the form:
-        
+
         UserTip(
             n=anInt
             tags=[list of tags],
             title=aString,
-            text="""
-            
+            text='''
+
             aString
-            
-            """)
-        
+
+            ''')
+
     After running this script, copy the result from the console to the
     <define tips> section of leoTips.py.
-    '''
+    """
     import requests
     url = 'https://api.github.com/repos/leo-editor/leo-editor/issues?labels=Tip&state='
-    
-    def get_tips(data):
+
+    def get_tips(data: Any) -> List[UserTip]:
         """get_tips - get tips from GitHub issues
         :param dict data: GitHub API issues list
         :return: list of Tips
@@ -80,24 +109,18 @@ def make_tips(c):
             lines = g.splitLines(body)
             for i, s in enumerate(lines):
                 if s.strip().lower().startswith('tags:'):
-                    lines = lines[:i] + lines[i+1:]
+                    lines = lines[:i] + lines[i + 1 :]
                     text = ''.join(lines).strip()
                     s = s.strip().rstrip('.').strip()
-                    s = s[len('tags:'):].strip()
+                    s = s[len('tags:') :].strip()
                     tags = [z.strip() for z in s.split(',')]
                     break
             else:
                 tags = []
                 text = body.strip()
-            tips.append(
-                UserTip(
-                    n=n,
-                    tags=tags,
-                    text=text.strip(),
-                    title=title.strip(),
-                ))
+            tips.append(UserTip(n=n, tags=tags, text=text.strip(), title=title.strip(),))
         return tips
-    
+
     g.cls()
     template = '''\
 UserTip(
@@ -105,21 +128,23 @@ UserTip(
     tags=%s,
     title="%s",
     text="""\
-    
+
 %s
 
 """),
 '''
-    template = g.adjustTripleString(template, c.tab_width)
-    for kind in ('open',): # 'closed':
-        data = requests.get(url+kind).json()
+    template = textwrap.dedent(template)
+    for kind in ('open',):  # 'closed':
+        data = requests.get(url + kind).json()
         for tip in get_tips(data):
-            tags = ["%s" % (z) for z in tip.tags or []]
+            tags = [f"{z}" for z in tip.tags or []]
             title = tip.title.lstrip('Tip:').lstrip('tip:').strip()
             print(template % (tip.n, tags, title, tip.text))
 #@+node:ekr.20180126052528.1: ** make_tip_nodes (leoTips.py)
-def make_tip_nodes(c):
-    '''Create a list of all tips as the last top-level node.'''
+#@@beautify
+
+def make_tip_nodes(c: Cmdr) -> None:
+    """Create a list of all tips as the last top-level node."""
     global tips
     root = c.lastTopLevel().insertAfter()
     root.h = 'User Tips'
@@ -129,14 +154,15 @@ def make_tip_nodes(c):
         p.h = tip.title
         p.b = tip.text
     if root.hasChildren():
-        root.h = '%s User Tips' % root.numberOfChildren()
+        root.h = f"{root.numberOfChildren()} User Tips"
         c.sortSiblings(p=root.firstChild())
     root.expand()
     c.selectPosition(root)
     c.redraw()
 #@-others
+
 # The global tips array.
-tips = [
+tips: List[UserTip] = [
 #@+<< define tips >>
 #@+node:ekr.20180121053422.1: ** << define tips >>
 #@@wrap
@@ -148,9 +174,9 @@ UserTip(
     tags=['Plugins'],
     title="Leo's most important plugins",
     text="""\
-    
+
 Become familiar with Leo's most important plugins:
-    
+
 - bookmarks.py manages bookmarks.
 - contextmenu.py shows a menu when when right-clicking.
 - mod_scripting.py supports @button and @command nodes.
@@ -166,28 +192,29 @@ UserTip(
     tags=['Power User',],
     title='Move clones to the last top-level node',
     text="""
-    
-Focus your attention of the task at hand by cloning nodes, including @file nodes, then moving those clones so they are the last top-level nodes in the outline.
 
-This allows you to work on nodes scattered throughout an outline without altering the structure of @file nodes.
+Focus your attention of the task at hand by cloning nodes,
+including @file nodes, then moving those clones so they are
+the last top-level nodes in the outline.
+
+This allows you to work on nodes scattered throughout an
+outline without altering the structure of @file nodes.
 
 """),
-
 #@+node:ekr.20180324065653.3: *4* myLeoSettings.leo
 UserTip(
     n=616,
     tags=['Settings',],
     title="Put personal settings myLeoSettings.leo",
     text="""\
-    
+
 Put your personal settings in myLeoSettings.leo, not leoSettings.leo.
 
-- The leo-settings-leo command opens leoSettings.leo.
-- The my-leo-settings-leo command opens myLeoSettings.leo.
+- The leo-settings command opens leoSettings.leo.
+- The my-leo-settings command opens myLeoSettings.leo.
 - Copy the desired settings nodes from leoSettings.leo to myLeoSettings.leo.
 
 """),
-
 #@+node:ekr.20180324065152.3: *4* Re @button make-md-toc
 UserTip(
     n=625,
@@ -202,7 +229,6 @@ You can then copy the text from the console to your document.
 The selected outline node should be an `@auto-md` node.
 
 """),
-
 #@+node:ekr.20180324073053.1: *3* Tips re Commands
 #@+node:ekr.20180324072156.1: *4* cff command
 UserTip(
@@ -211,12 +237,13 @@ UserTip(
     title="The clone-find commands gather nodes matching a pattern",
     text="""
 
-The cff command (aka clone-find-flattened) prompts for a search pattern, then clones all matching nodes so they are the children of a new last top-level node.
+The cff command (aka clone-find-flattened) prompts for a
+search pattern, then clones all matching nodes so they are
+the children of a new last top-level node.
 
 This is a great way to study code.
 
 """),
-
 #@+node:ekr.20180324065153.1: *4* beautify command & @nobeautify
 UserTip(
     n=623,
@@ -234,12 +261,13 @@ UserTip(
     title="The cffm command gathers marked outline nodes",
     text="""
 
-The cffm command (aka clone-find-flattened-marked) clones all marked nodes as a children of a new node, created as the last top-level node.
+The cffm command (aka clone-find-flattened-marked) clones
+all marked nodes as a children of a new node, created as the
+last top-level node.
 
 Use this to gather nodes throughout an outline.
 
 """),
-
 #@+node:ekr.20180324072541.1: *4* find-quick-selected command
 UserTip(
     n=607,
@@ -250,20 +278,18 @@ UserTip(
 The find-quick-selected (Ctrl-Shift-F) command finds all nodes containing the selected text.
 
 """),
-
 #@+node:ekr.20180324072904.1: *4* goto-next-clone command
 UserTip(
     n=0,
     tags=['Commands', 'Power User',],
     title='Alt-N (goto-next-clone) finds "primary" clones',
     text="""
-    
+
 Use Alt-N to cycle through the clones of the present cloned node.
 
 This is a fast way of finding the clone whose ancestor is an @<file> node.
-    
-"""),
 
+"""),
 #@+node:ekr.20180527052858.1: *4* help-*
 UserTip(
     n=0,
@@ -286,8 +312,10 @@ UserTip(
     tags=['Commands',],
     title='leo-* commands open common .leo files',
     text="""
-    
-You can open files such as CheatSheet.leo, quickstart.leo, leoSettings.leo, myLeoSettings.leo and scripts.leo with commands starting with 'leo-'.
+
+You can open files such as CheatSheet.leo, quickstart.leo,
+leoSettings.leo, myLeoSettings.leo and scripts.leo with
+commands starting with 'leo-'.
 
 <Alt-X>leo-<tab> shows the complete list.
 
@@ -302,7 +330,6 @@ UserTip(
 The parse-body command parses p.b (the body text of the selected node) into separate nodes.
 
 """),
-
 #@+node:ekr.20180324065153.2: *4* pylint command
 UserTip(
     n=622,
@@ -322,7 +349,7 @@ UserTip(
     tags=['Power User',],
     title='Ctrl-P (repeat-complex-command)',
     text="""
-    
+
 Ctrl-P re-executes the last command made from the minibuffer.
 
 You can use this to avoid having to define key bindings.
@@ -332,7 +359,6 @@ For example, instead of pressing an @button button, execute its command from the
 Now you can re-execute the button using Ctrl-P.
 
 """),
-
 #@+node:ekr.20180324065153.3: *4* rst3 command
 UserTip(
     n=621,
@@ -344,7 +370,6 @@ UserTip(
 <p>See <a href="http://leoeditor.com/tutorial-rst3.html">Leo's rst3 tutorial.</a></p>
 
 </html>"""),
-
 #@+node:ekr.20180324072625.1: *4* sort-siblings command
 UserTip(
     n=605,
@@ -355,7 +380,6 @@ UserTip(
 The sort-siblings (Alt-A) command sorts all the child nodes of their parent, or all top-level nodes.
 
 """),
-
 #@+node:ekr.20180324073210.1: *3* Tips re Scripting
 #@+node:ekr.20180324065152.1: *4* Clearing the log window
 UserTip(
@@ -363,8 +387,11 @@ UserTip(
     tags=['Scripting'],
     title="Clearing the Log window",
     text="""\
-    
-When developing scripts that use Log window to display results, it is sometimes useful to clear Log window by inserting the following two lines at the beginning of your script:
+
+When developing scripts that use Log window to display
+results, it is sometimes useful to clear Log window by
+inserting the following two lines at the beginning of your
+script:
 
     c.frame.log.selectTab('Log')
     c.frame.log.clearLog()
@@ -376,16 +403,17 @@ UserTip(
     tags=['Scripting', 'Debugging', 'Beginner'],
     title="<html>The g.callers() function",
     text="""
-<p>g.callers() returns the last n callers (default 4) callers of a function or method. The verbose option shows each caller on a separate line.  For example:</p>
+<p>g.callers() returns the last n callers (default 4) callers of a function or method.
+The verbose option shows each caller on a separate line.
+For example:</p>
 
 <p><pre>    g.trace(g.callers())</pre></p>
 
 <p>You must
 <a href="http://leoeditor.com/running.html#running-leo-from-a-console-window">
 run Leo from a console</a> for this to work.</p>
- 
-</html>"""),
 
+</html>"""),
 #@+node:ekr.20180324072527.1: *4* g.pdb
 UserTip(
     n=608,
@@ -401,7 +429,6 @@ adapted for Leo.</p>
 run Leo from a console</a> for this to work.</p>
 
 </html>"""),
-
 #@+node:ekr.20180324072513.1: *4* g.trace
 UserTip(
     n=609,
@@ -440,7 +467,7 @@ UserTip(
     tags=['Scripting', 'Tutorial'],
     title="@button nodes create commands",
     text="""
-    
+
 For example, `@button my-command` creates the `my-command` button and the `my-command` command.
 
 Within `@button` scripts, c.p is the presently selected outline node.
@@ -448,56 +475,38 @@ Within `@button` scripts, c.p is the presently selected outline node.
 As a result, @button nodes bring scripts to data.
 
 """),
-
-#@+node:ekr.20180324065153.5: *4* Re @test
-UserTip(
-    n=619,
-    tags=['Tutorial', 'Testing', 'Scripting'],
-    title="<html>@test nodes create unit tests",
-    text="""
-
-<p><a href="http://leoeditor.com/tutorial-basics.html#test-nodes">@test nodes</a>
-automatically convert the body to a subclass of
-<a href="https://docs.python.org/3/library/unittest.html#test-cases">unittest.TestCase.</a>
-</p>
-
-<p>Leo's run-* commands execute unit tests.</p>
-
-</html>"""),
 #@+node:ekr.20180324065152.2: *4* Section refs vs @others
 UserTip(
     n=626,
     tags=[],
     title="Use section references sparingly",
-    text="""
+    text=f"""
 
-Within scripts, use section references only when code must be placed exactly. Here is a common pattern for @file nodes for python files:
+Within scripts, use section references only when code must
+be placed exactly. Here is a common pattern for @file nodes
+for python files:
 
     @first # -*- coding: utf-8 -*-
-    %s
-    %s
-
-""" % (g.angleBrackets('imports'), '@others')),
+    {g.angleBrackets('imports')}
+    {'@others'}
 
+"""),
 #@+node:ekr.20180324085629.1: *4* Use section refs to avoid "one @others per node" rule
 UserTip(
     n=0,
     tags=['Scripting',],
     title='Use section refs to avoid one @others per node rule',
-    text="""
+    text=f"""
 
-Nodes can have at most one @others directive. You can work around this restriction as follows:
+Nodes can have at most one {at_s}others directive. You can work around this restriction as follows:
 
-    %(at)sfile myFile.py
-    %(at)sothers
-    %(start)s organizer %(end)s
+    {at_s}file myFile.py
+    {at_s}others
+    {ref1_s} organizer {ref2_s}
 
-where the body of the %(start)s organizer %(end)s node contains just @others.""" % {
-    'at': "@",
-    'end': ">>",
-    'start': "<<",
-}),
+where the body of the {ref1_s} organizer {ref2_s} node contains just {at_s}others.
 
+"""),
 #@+node:ekr.20180324073458.1: *3* Tips re Work flow
 #@+node:ekr.20180324065153.4: *4* Abbreviations
 UserTip(
@@ -516,7 +525,6 @@ can correct spelling mistakes, expand to multiple lines or even trees of nodes.
 can prompt for values to be substituted within the abbreviation.</p>
 
 </html>"""),
-
 #@+node:ekr.20180324072110.1: *4* Clones
 UserTip(
     n=615,
@@ -530,7 +538,6 @@ are "live" copies of the node itself and all its descendants.</p>
 <p>Clones are a unique feature of Leo.</p>
 
 </html>"""),
-
 #@+node:ekr.20180324072128.1: *4* Don't remember command names
 UserTip(
     n=614,
@@ -543,19 +550,20 @@ To execute a command, type `Alt-X` followed by the first few characters of comma
 The list of commands matching what you have typed appears.
 
 """),
-
 #@+node:ekr.20180324065145.1: *4* How to assign shortcuts to scripts
 UserTip(
     n=629,
     tags=['Scripting'],
     title="How to assign shortcuts to scripts",
     text="""
-    
+
 You can have a personal shortcut to run script while developing it.
 
 For example: put `@key=Alt-4` in headline.
 
-If your script grows to several subnodes, you won't have to select top node every time you wish to run script. It would be enough to just press your universal shortcut.
+If your script grows to several subnodes, you won't have to
+select top node every time you wish to run script. It would
+be enough to just press your universal shortcut.
 
 """),
 #@+node:ekr.20180324072951.1: *4* How to find documenation
@@ -564,31 +572,29 @@ UserTip(
     tags=['Beginner',],
     title='How to find documentation',
     text="""
-    
+
 Just search LeoDocs.leo.
 
 """),
-
 #@+node:ekr.20180324072812.1: *4* How to find settings
 UserTip(
     n=0,
     tags=['Settings',],
     title="How to find settings",
     text="""
-    
+
 Just search leoSettings.leo.
 
 leoSettings.leo contains the defaults for all of Leo's settings, with documentation for each.
 
 """),
-
 #@+node:ekr.20180312101254.1: *4* How to find your @command nodes
 UserTip(
     n=0,
     tags=['Settings',],
     title="How to find all your @command nodes",
     text="""
-    
+
 <alt-x>@c<tab> shows all the @command nodes in effect for the present outline, no matter where defined.
 
 myLeoSettings.leo can define *common* @command nodes that apply to all outlines.
@@ -603,9 +609,12 @@ UserTip(
     title="How to minimize panes",
     text="""
 
-Middle mouse click on the window divider (splitter) to minimize all windows to the left of vertical splitters and below horizontal splitters.
+Middle mouse click on the window divider (splitter) to
+minimize all windows to the left of vertical splitters and
+below horizontal splitters.
 
-Using this with "Toggle Split Direction" allows a maximized body, tree or even log window.
+Using this with "Toggle Split Direction" allows a maximized
+body, tree or even log window.
 
 """),
 #@+node:ekr.20180503082333.1: *4* How to see all @command commands
@@ -621,6 +630,7 @@ Note: myLeoSettings.leo can define common @command nodes that apply to all outli
 
 """),
 #@-others
+#@@beautify
 #@-<< define tips >>
 ]
 #@@language python
