@@ -42,9 +42,10 @@ leoVersion.version:     Leo's version number.
 # 6.6.2:  May 25, 2022.
 # 6.6.3:  July 22, 2022.
 # 6.7.0:  September 26, 2022.
+# 6.7.1:  November 8, 2022.
 #@-<< version dates >>
-version = '6.7.0'
-static_date = 'September 26, 2022'
+version = '6.7.1'
+static_date = 'November 11, 2022'
 #@@language python
 #@@tabwidth -4
 #@-leo
