@@ -1,7 +1,9 @@
 #! /usr/bin/env python
+# -*- coding: utf-8 -*-
 #@+leo-ver=5-thin
 #@+node:ekr.20031218072017.2605: * @file runLeo.py
 #@@first
+#@@first
 """Entry point for Leo in Python."""
 #@+<< imports and inits >>
 #@+node:ekr.20080921091311.1: ** << imports and inits >> (runLeo.py)
@@ -81,7 +83,7 @@ def profile_leo():
 
 prof = profile_leo
 #@+node:ekr.20120219154958.10499: ** run (runLeo.py)
-def run(fileName=None, pymacs=None, *args, **keywords):
+def run(fileName=None, pymacs: bool=None, *args, **keywords):
     """Initialize and run Leo"""
     # #1403: sys.excepthook doesn't help.
     # sys.excepthook = leo_excepthook
