@@ -2,39 +2,34 @@
 #@+node:ekr.20170428084208.398: * @file ../external/npyscreen/wgwidget.py
 #!/usr/bin/python
 # pylint: disable=no-member,access-member-before-definition
+# type: ignore
 #@+<< wgwidget imports >>
 #@+node:ekr.20170428084208.399: ** << wgwidget imports >>
-import leo.core.leoGlobals as g
-# import leo.plugins.cursesGui2 as cursesGui2
-assert g
-
-# import codecs
 import copy
 import curses
 import curses.ascii
 import re
-# import string
 import sys
-#import curses.wrapper
 import weakref
 from . import npysGlobalOptions as GlobalOptions
 from . import wgwidget_proto
 import locale
-# import warnings
 from .globals import DEBUG
 # experimental
 from .eveventhandler import EventHandler
+from leo.core import leoGlobals as g
+assert g
 #@-<< wgwidget imports >>
 #@+<< wgwidgets data >>
 #@+node:ekr.20170429213125.1: ** << wgwidgets data >>
-EXITED_DOWN  =  1
-EXITED_UP    = -1
-EXITED_LEFT  = -2
-EXITED_RIGHT =  2
-EXITED_ESCAPE= 127
+EXITED_DOWN = 1
+EXITED_UP = -1
+EXITED_LEFT = -2
+EXITED_RIGHT = 2
+EXITED_ESCAPE = 127
 EXITED_MOUSE = 130
-SETMAX       = 'SETMAX'
-RAISEERROR   = 'RAISEERROR'
+SETMAX = 'SETMAX'
+RAISEERROR = 'RAISEERROR'
 ALLOW_NEW_INPUT = True
 
 TEST_SETTINGS = {
@@ -68,7 +63,7 @@ class NotEnoughSpaceForWidget(Exception):
     pass
 
 #@+node:ekr.20170428084208.404: ** class InputHandler (wgwidget.py)
-class InputHandler(object):
+class InputHandler:
     "An object that can handle user input"
 
     #@+others
@@ -84,7 +79,7 @@ class InputHandler(object):
         trace = False
         trace_entry = True
         trace_parent = False
-        
+
         def tell(f):
             pattern = r'<bound method ([\w\.]*\.)?(\w+) of <([\w\.]*\.)?(\w+) object at (.+)>>'
             m = re.match(pattern, repr(f))
@@ -122,7 +117,7 @@ class InputHandler(object):
             f(i)
             return True
         for test, handler in getattr(self, 'complex_handlers', []):
-            if test(i): # was is not False.
+            if test(i):  # was is not False.
                 if trace: g.trace('complex: %3s %s' % (i, tell(handler)))
                 return handler(i)
         if parent_widget and hasattr(parent_widget, 'handle_input'):
@@ -153,18 +148,18 @@ class InputHandler(object):
         """
         #called in __init__
         self.handlers = {
-            curses.ascii.NL:     self.h_exit_down,
-            curses.ascii.CR:     self.h_exit_down,
-            curses.ascii.TAB:    self.h_exit_down,
-            curses.KEY_BTAB:     self.h_exit_up,
-            curses.KEY_DOWN:     self.h_exit_down,
-            curses.KEY_UP:       self.h_exit_up,
-            curses.KEY_LEFT:     self.h_exit_left,
-            curses.KEY_RIGHT:    self.h_exit_right,
-            "^P":                self.h_exit_up,
-            "^N":                self.h_exit_down,
-            curses.ascii.ESC:    self.h_exit_escape,
-            curses.KEY_MOUSE:    self.h_exit_mouse,
+            curses.ascii.NL: self.h_exit_down,
+            curses.ascii.CR: self.h_exit_down,
+            curses.ascii.TAB: self.h_exit_down,
+            curses.KEY_BTAB: self.h_exit_up,
+            curses.KEY_DOWN: self.h_exit_down,
+            curses.KEY_UP: self.h_exit_up,
+            curses.KEY_LEFT: self.h_exit_left,
+            curses.KEY_RIGHT: self.h_exit_right,
+            "^P": self.h_exit_up,
+            "^N": self.h_exit_down,
+            curses.ascii.ESC: self.h_exit_escape,
+            curses.KEY_MOUSE: self.h_exit_mouse,
         }
         self.complex_handlers = []
 
@@ -252,8 +247,8 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
     #@+node:ekr.20170428084208.412: *3* Widget.__init__
     def __init__(self, screen,
             relx=0, rely=0, name=None, value=None,
-            width = False, height = False,
-            max_height = False, max_width=False,
+            width=False, height=False,
+            max_height=False, max_width=False,
             editable=True,
             hidden=False,
             color='DEFAULT',
@@ -275,11 +270,12 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         value_changed_callback - should be None or a Function.  If it is a function, it will have be called when the value changes
                                and passed the keyword argument widget=self.
         """
-        self.check_value_change=check_value_change
-        self.check_cursor_move =check_cursor_move
+        self.check_value_change = check_value_change
+        self.check_cursor_move = check_cursor_move
         self.hidden = hidden
-        self.interested_in_mouse_even_when_not_editable = False# used only for rare widgets to allow user to click
-                                                        # even if can't actually select the widget.  See mutt-style forms
+        self.interested_in_mouse_even_when_not_editable = False
+            # used only for rare widgets to allow user to click
+            # even if can't actually select the widget.  See mutt-style forms
 
         try:
             self.parent = weakref.proxy(screen)
@@ -290,7 +286,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         #self.relx = relx
         #self.rely = rely
         self.color = color
-        self.encoding = 'utf-8'#locale.getpreferredencoding()
+        self.encoding = 'utf-8'  #locale.getpreferredencoding()
         if GlobalOptions.ASCII_ONLY or locale.getpreferredencoding() == 'US-ASCII':
             self._force_ascii = True
         else:
@@ -309,20 +305,20 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         try:
             self.name
         except Exception:
-            self.name=name
+            self.name = name
 
-        self.request_width =  width     # widgets should honour if possible
-        self.request_height = height    # widgets should honour if possible
+        self.request_width = width  # widgets should honour if possible
+        self.request_height = height  # widgets should honour if possible
 
         self.max_height = max_height
         self.max_width = max_width
 
         self.set_size()
 
-        self.editing = False        # Change to true during an edit
+        self.editing = False  # Change to true during an edit
 
         self.editable = editable
-        if self.parent.curses_pad.getmaxyx()[0]-1 == self.rely: self.on_last_line = True
+        if self.parent.curses_pad.getmaxyx()[0] - 1 == self.rely: self.on_last_line = True
         else: self.on_last_line = False
 
         if value_changed_callback:
@@ -337,7 +333,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         if trace: g.trace('BEGIN')
         if not self.parent.editing:
             _i_set_parent_editing = True
-            self.parent.editing   = True
+            self.parent.editing = True
         else:
             _i_set_parent_editing = False
         while self.editing and self.parent.editing:
@@ -348,7 +344,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
             self.parent.editing = False
         if trace: g.trace('END')
         if self.editing:
-            self.editing    = False
+            self.editing = False
             self.how_exited = True
     #@+node:ekr.20170429213619.5: *3* Widget._get_ch
     def _get_ch(self):
@@ -362,7 +358,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         # but everything that follows could be in the except clause above.
 
             # Try to read utf-8 if possible.
-        _stored_bytes =[]
+        _stored_bytes = []
         self._last_get_ch_was_unicode = True
         global ALLOW_NEW_INPUT
         if ALLOW_NEW_INPUT == True and locale.getpreferredencoding() == 'UTF-8':
@@ -379,20 +375,20 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
             # all of the subsequent bytes should be in the range 128 - 191,
             # but we'll risk not checking...
             elif 194 <= ch <= 223:
-                    # 2 bytes
-                    _stored_bytes.append(ch)
-                    _stored_bytes.append(self.parent.curses_pad.getch())
+                # 2 bytes
+                _stored_bytes.append(ch)
+                _stored_bytes.append(self.parent.curses_pad.getch())
             elif 224 <= ch <= 239:
-                    # 3 bytes
-                    _stored_bytes.append(ch)
-                    _stored_bytes.append(self.parent.curses_pad.getch())
-                    _stored_bytes.append(self.parent.curses_pad.getch())
+                # 3 bytes
+                _stored_bytes.append(ch)
+                _stored_bytes.append(self.parent.curses_pad.getch())
+                _stored_bytes.append(self.parent.curses_pad.getch())
             elif 240 <= ch <= 244:
-                    # 4 bytes
-                    _stored_bytes.append(ch)
-                    _stored_bytes.append(self.parent.curses_pad.getch())
-                    _stored_bytes.append(self.parent.curses_pad.getch())
-                    _stored_bytes.append(self.parent.curses_pad.getch())
+                # 4 bytes
+                _stored_bytes.append(ch)
+                _stored_bytes.append(self.parent.curses_pad.getch())
+                _stored_bytes.append(self.parent.curses_pad.getch())
+                _stored_bytes.append(self.parent.curses_pad.getch())
             elif ch >= 245:
                 # probably a control character
                 self._last_get_ch_was_unicode = False
@@ -435,7 +431,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         "Internal Method. This will be the method called when the terminal resizes."
         self._move_widget_on_terminal_resize()
         self._recalculate_size()
-        if self.parent.curses_pad.getmaxyx()[0]-1 == self.rely: self.on_last_line = True
+        if self.parent.curses_pad.getmaxyx()[0] - 1 == self.rely: self.on_last_line = True
         else: self.on_last_line = False
         self.resize()
         self.when_resized()
@@ -456,7 +452,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         calculate the screen area they need, returning either y,x, or 0,0 if
         they want all the screen they can.  However, do not use this to say how
         big a given widget is ... use .height and .width instead"""
-        return 0,0
+        return 0, 0
     #@+node:ekr.20170428084208.427: *3* Widget.clear
     def clear(self, usechar=' '):
         """Blank the screen area used by this widget, ready for redrawing"""
@@ -473,10 +469,10 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
             if self.do_colors():
                 color = self.parent.theme_manager.findPair(self, self.parent.color)
                 for y in range(self.height):
-                    pad.addstr(self.rely+y, self.relx, s, color)
+                    pad.addstr(self.rely + y, self.relx, s, color)
             else:
                 for y in range(self.height):
-                    pad.addstr(self.rely+y, self.relx, s)
+                    pad.addstr(self.rely + y, self.relx, s)
             # Old code
             # if self.do_colors():
                 # self.parent.curses_pad.addstr(
@@ -520,7 +516,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
     #@+node:ekr.20170429213619.1: *3* Widget.edit
     def edit(self):
         """Allow the user to edit the widget: ie. start handling keypresses."""
-        g.trace('===== (Widget)')
+        # g.trace('===== (Widget)')
         self.editing = 1
         self._pre_edit()
         self._edit_loop()
@@ -550,7 +546,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
                 ch2 = self.parent.curses_pad.getch()
                 if ch2 != -1:
                     ch = curses.ascii.alt(ch2)
-                self.parent.curses_pad.timeout(-1) # back to blocking mode
+                self.parent.curses_pad.timeout(-1)  # back to blocking mode
                 #curses.flushinp()
         elif (TEST_SETTINGS['INPUT_GENERATOR']):
             self._last_get_ch_was_unicode = True
@@ -584,7 +580,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         self.try_adjust_widgets()
     #@+node:ekr.20170428084208.426: *3* Widget.get_editable
     def get_editable(self):
-        return(self._is_editable)
+        return (self._is_editable)
     #@+node:ekr.20170429213619.10: *3* Widget.handle_mouse_event
     def handle_mouse_event(self, mouse_event):
         # mouse_id, x, y, z, bstate = mouse_event
@@ -594,7 +590,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         mouse_id, x, y, z, bstate = mouse_event
         x += self.parent.show_from_x
         y += self.parent.show_from_y
-        rel_y       = y - self.rely - self.parent.show_aty
+        rel_y = y - self.rely - self.parent.show_aty
         rel_x = x - self.relx - self.parent.show_atx
         return (mouse_id, rel_x, rel_y, z, bstate)
 
@@ -608,8 +604,8 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         mouse_id, x, y, z, bstate = mouse_event
         x += self.parent.show_from_x
         y += self.parent.show_from_y
-        if self.relx <= x <= self.relx + self.width-1 + self.parent.show_atx:
-            if self.rely  <= y <= self.rely + self.height-1 + self.parent.show_aty:
+        if self.relx <= x <= self.relx + self.width - 1 + self.parent.show_atx:
+            if self.rely <= y <= self.rely + self.height - 1 + self.parent.show_aty:
                 return True
         return False
     #@+node:ekr.20170428084208.416: *3* Widget.resize
@@ -757,7 +753,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         ny, nx = self.calculate_area_needed()
 
         max_height = self.max_height
-        max_width  = self.max_width
+        max_width = self.max_width
         # What to do if max_height or max_width is negative
         if max_height not in (None, False) and max_height < 0:
             max_height = my + max_height
@@ -786,12 +782,12 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         #    raise Exception("Not enough space for widget")
 
 
-        if nx > 0:                 # if a minimum space is specified....
-            if mx >= nx:           # if max width is greater than needed space
-                self.width = nx    # width is needed space
+        if nx > 0:  # if a minimum space is specified....
+            if mx >= nx:  # if max width is greater than needed space
+                self.width = nx  # width is needed space
             else:
-                self.width = RAISEERROR    # else raise an error
-        elif self.max_width:       # otherwise if a max width is speciied
+                self.width = RAISEERROR  # else raise an error
+        elif self.max_width:  # otherwise if a max width is speciied
             if max_width <= mx:
                 self.width = max_width
             else:
@@ -800,7 +796,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
             self.width = self.request_width or mx
         if self.height == RAISEERROR or self.width == RAISEERROR:
             # Not enough space for widget
-            raise NotEnoughSpaceForWidget("Not enough space: max y and x = %s , %s. Height and Width = %s , %s " % (my, mx, self.height, self.width) ) # unsafe. Need to add error here.
+            raise NotEnoughSpaceForWidget("Not enough space: max y and x = %s , %s. Height and Width = %s , %s " % (my, mx, self.height, self.width))  # unsafe. Need to add error here.
     #@+node:ekr.20170428084208.420: *3* Widget.space_available
     def space_available(self):
         """The space available left on the screen, returned as rows, columns"""
@@ -808,7 +804,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
             y, x = self.parent.useable_space(self.rely, self.relx)
         else:
             y, x = self.parent.widget_useable_space(self.rely, self.relx)
-        return y,x
+        return y, x
     #@+node:ekr.20170429213619.6: *3* Widget.try_adjust_widgets
     def try_adjust_widgets(self):
         if hasattr(self.parent, "adjust_widgets"):
@@ -834,8 +830,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
         actually refresh the curses display, since this should be done as
         little as possible. This base widget puts nothing on screen.
         """
-        trace = False and not g.unitTesting
-        if trace: g.trace('===== Widget', g.callers())
+        # g.trace('===== Widget', g.callers())
         if self.hidden:
             self.clear()
             return True
@@ -889,7 +884,7 @@ class Widget(InputHandler, wgwidget_proto._LinePrinter, EventHandler):
     def when_value_edited(self):
         """
         Called when the user edits the value of the widget.
-        
+
         Will usually also be called the first time that the user edits the
         widget.
         """
@@ -917,13 +912,13 @@ class DummyWidget(Widget):
         else: self._is_editable = False
     #@+node:ekr.20170428084208.433: *3* DummyWidget.get_editable
     def get_editable(self):
-        return(self._is_editable)
+        return (self._is_editable)
     #@+node:ekr.20170428084208.434: *3* DummyWidget.clear
     def clear(self, usechar=' '):
         pass
     #@+node:ekr.20170428084208.435: *3* DummyWidget.calculate_area_needed
     def calculate_area_needed(self):
-        return 0,0
+        return 0, 0
 
 
     #@-others
