@@ -13,6 +13,7 @@ Revised by EKR February 6-7, 2017.
 #@+<< demo.py imports >>
 #@+node:ekr.20170128213103.3: **  << demo.py imports >>
 import random
+from typing import List
 from leo.core import leoGlobals as g
 from leo.plugins import qt_events
 from leo.core.leoQt import QtCore, QtGui, QtWidgets
@@ -20,6 +21,7 @@ from leo.core.leoQt import QtCore, QtGui, QtWidgets
 # Fail fast, right after all imports.
 g.assertUi('qt')  # May raise g.UiTypeException, caught by the plugins manager.
 #@-<< demo.py imports >>
+
 # pylint: disable=no-member,not-callable
 #@@language python
 #@@tabwidth -4
@@ -97,6 +99,7 @@ class Demo:
         self.script_i = 0
         # A list of strings (scripts). Scripts are removed when executed.
         self.script_list = []
+        self.speed: float = None
         self.user_dict = {}  # For use by scripts.
         self.widgets = []  # References to all widgets created by this class.
         # Init...
@@ -210,8 +213,7 @@ class Demo:
             script = self.script_list[self.script_i]
             self.setup_script()
             self.exec_node(script)
-            self.script_i += 1
-                # Restore invariant, and make net change = -1.
+            self.script_i += 1  # Restore invariant, and make net change = -1.
             self.teardown_script()
 
     prev_command = prev
@@ -313,7 +315,7 @@ class Demo:
         Return a list of strings.
         """
         aList = []
-        lines = []
+        lines: List[str] = []
         for s in g.splitLines(script_string):
             if s.startswith(delim):
                 if lines:
@@ -331,7 +333,7 @@ class Demo:
             aList.append(''.join(lines))
         return aList
     #@+node:ekr.20170128213103.43: *4* demo.wait & key_wait
-    def key_wait(self, speed=None, n1=None, n2=None):
+    def key_wait(self, speed: float=None, n1=None, n2=None):
         """Wait for an interval between n1 and n2, in seconds."""
         if n1 is None:
             n1 = self.n1
@@ -499,7 +501,7 @@ class Demo:
             p = g.findNodeAnywhere(c, headline)
         return p
     #@+node:ekr.20170211045602.1: *4* demo.insert_node
-    def insert_node(self, headline, end=True, keys=False, speed=None):
+    def insert_node(self, headline, end=True, keys=False, speed: float=None):
         """Helper for inserting a node."""
         c = self.c
         p = c.insertHeadline()
@@ -713,7 +715,7 @@ class Demo:
     #@-others
 #@+node:ekr.20170208045907.1: ** Graphics classes & helpers
 #@+node:ekr.20170206203005.1: *3*  class Label (QLabel)
-class Label(QtWidgets.QLabel):
+class Label(QtWidgets.QLabel):  # type:ignore
     """A class for user-defined callouts in demo.py."""
 
     def __init__(self, text,
@@ -786,8 +788,7 @@ class Callout(Label):
         # Do this *after* initing the base class.
         demo.set_position(w, position or 'center')
 #@+node:ekr.20170208065111.1: *3* class Image (QLabel)
-class Image(QtWidgets.QLabel):
-
+class Image(QtWidgets.QLabel):  # type:ignore
     def __init__(self, fn,
         pane=None, magnification=None, position=None, size=None):
         """Image.__init__."""
@@ -833,8 +834,8 @@ class Image(QtWidgets.QLabel):
             demo.set_position(widget, position)
         widget.setPixmap(pixmap)
     #@-others
-#@+node:ekr.20170208095240.1: *3* class Text (QTextEdit)
-class Text(QtWidgets.QPlainTextEdit):
+#@+node:ekr.20170208095240.1: *3* class Text (QPlainTextEdit)
+class Text(QtWidgets.QPlainTextEdit):  # type:ignore
 
     def __init__(self, text,
         font=None, pane=None, position=None, size=None, stylesheet=None
