@@ -3,11 +3,11 @@
 #@+node:ekr.20140831085423.18598: * @file ../plugins/qt_text.py
 #@@first
 """Text classes for the Qt version of Leo"""
-#@+<< imports qt_text.py >>
-#@+node:ekr.20220416085845.1: ** << imports qt_text.py >>
+#@+<< qt_text imports >>
+#@+node:ekr.20220416085845.1: ** << qt_text imports >>
 import time
 assert time
-from typing import Any, Callable, Dict, List, Tuple
+from typing import Any, Callable, Dict, List, Optional, Tuple
 from typing import TYPE_CHECKING
 from leo.core import leoGlobals as g
 from leo.core.leoQt import isQt6, QtCore, QtGui, Qsci, QtWidgets
@@ -15,20 +15,21 @@ from leo.core.leoQt import ContextMenuPolicy, Key, KeyboardModifier, Modifier
 from leo.core.leoQt import MouseButton, MoveMode, MoveOperation
 from leo.core.leoQt import Shadow, Shape, SliderAction, SolidLine, WindowType, WrapMode
 
-#@-<< imports qt_text.py >>
-#@+<< type aliases qt_text.py >>
-#@+node:ekr.20220416085945.1: ** << type aliases qt_text.py >>
-if TYPE_CHECKING:  # Always False at runtime.
+#@-<< qt_text imports >>
+#@+<< qt_text annotations >>
+#@+node:ekr.20220416085945.1: ** << qt_text annotations >>
+if TYPE_CHECKING:  # pragma: no cover
     from leo.core.leoCommands import Commands as Cmdr
-    # from leo.core.leoNodes import Position as Pos
+    from leo.core.leoGui import LeoKeyEvent as Event
+    from leo.plugins.qt_text import QTextEditWrapper as Wrapper  # pylint: disable=import-self
 else:
     Cmdr = Any
-
-Event = Any
-Index = Any  # For now, really Union[int, str], but that creates type-checking problems.
+    Event = Any
+    Wrapper = Any
 Widget = Any
-Wrapper = Any
-#@-<< type aliases qt_text.py >>
+
+MousePressEvent = Any
+#@-<< qt_text annotations >>
 
 FullWidthSelection = 0x06000  # works for both Qt5 and Qt6
 QColor = QtGui.QColor
@@ -244,12 +245,10 @@ class QTextMixin:
         self.setAllText(s2 + s)
         self.setInsertPoint(len(s2))
     #@+node:ekr.20140901141402.18706: *5* qtm.delete
-    def delete(self, i: Index, j: Index=None) -> None:
+    def delete(self, i: int, j: int=None) -> None:
         """QTextMixin"""
-        i = self.toPythonIndex(i)
         if j is None:
             j = i + 1
-        j = self.toPythonIndex(j)
         # This allows subclasses to use this base class method.
         if i > j:
             i, j = j, i
@@ -269,11 +268,9 @@ class QTextMixin:
         # https://bugs.launchpad.net/leo-editor/+bug/979142
         # https://bugs.launchpad.net/leo-editor/+bug/971166
         s = self.getAllText()
-        i = self.toPythonIndex(i)
-        j = self.toPythonIndex(j)
         return s[i:j]
-    #@+node:ekr.20140901062324.18704: *5* qtm.getLastPosition & getLength
-    def getLastPosition(self, s: str=None) -> int:
+    #@+node:ekr.20140901062324.18704: *5* qtm.getLastIndex & getLength
+    def getLastIndex(self, s: str=None) -> int:
         """QTextMixin"""
         return len(self.getAllText()) if s is None else len(s)
 
@@ -283,16 +280,15 @@ class QTextMixin:
     #@+node:ekr.20140901062324.18705: *5* qtm.getSelectedText
     def getSelectedText(self) -> str:
         """QTextMixin"""
-        i, j = self.getSelectionRange()
+        i, j = self.getSelectionRange()  # Returns (int, int)
         if i == j:
             return ''
         s = self.getAllText()
         return s[i:j]
     #@+node:ekr.20140901141402.18702: *5* qtm.insert
-    def insert(self, i: Index, s: str) -> int:
+    def insert(self, i: int, s: str) -> int:
         """QTextMixin"""
         s2 = self.getAllText()
-        i = self.toPythonIndex(i)
         self.setAllText(s2[:i] + s + s2[i:])
         self.setInsertPoint(i + len(s))
         return i
@@ -305,27 +301,19 @@ class QTextMixin:
     def selectAllText(self, s: str=None) -> None:
         """QTextMixin."""
         self.setSelectionRange(0, self.getLength(s))
-    #@+node:ekr.20140901141402.18710: *5* qtm.toPythonIndex
-    def toPythonIndex(self, index: Index, s: str=None) -> int:
-        """QTextMixin"""
-        if s is None:
-            s = self.getAllText()
-        i = g.toPythonIndex(s, index)
-        return i
     #@+node:ekr.20140901141402.18704: *5* qtm.toPythonIndexRowCol
-    def toPythonIndexRowCol(self, index: int) -> Tuple[int, int, int]:
+    def toPythonIndexRowCol(self, index: int) -> Tuple[int, int]:
         """QTextMixin"""
         s = self.getAllText()
-        i = self.toPythonIndex(index)
-        row, col = g.convertPythonIndexToRowCol(s, i)
-        return i, row, col
+        row, col = g.convertPythonIndexToRowCol(s, index)
+        return row, col
     #@+node:ekr.20140901062324.18729: *4* qtm.rememberSelectionAndScroll
     def rememberSelectionAndScroll(self) -> None:
 
         w = self
         v = self.c.p.v  # Always accurate.
         v.insertSpot = w.getInsertPoint()
-        i, j = w.getSelectionRange()
+        i, j = w.getSelectionRange()  # Returns (int, int)
         if i > j:
             i, j = j, i
         assert i <= j
@@ -361,6 +349,21 @@ class QLineEditWrapper(QTextMixin):
         """
         return True
     #@+node:ekr.20110605121601.18118: *3* qlew.Widget-specific overrides
+    #@+node:ekr.20220911105050.1: *4* qlew: do-nothings
+    def flashCharacter(self, i: int, bg: str='white', fg: str='red', flashes: int=3, delay: int=75) -> None:
+        pass
+
+    def getXScrollPosition(self) -> int:
+        return 0
+
+    def getYScrollPosition(self) -> int:
+        return 0
+
+    def setXScrollPosition(self, i: int) -> None:
+        pass
+
+    def setYScrollPosition(self, i: int) -> None:
+        pass
     #@+node:ekr.20110605121601.18120: *4* qlew.getAllText
     def getAllText(self) -> str:
         """QHeadlineWrapper."""
@@ -387,9 +390,6 @@ class QLineEditWrapper(QTextMixin):
                 i = j = w.cursorPosition()
             return i, j
         return 0, 0
-    #@+node:ekr.20210104122029.1: *4* qlew.getYScrollPosition
-    def getYScrollPosition(self) -> int:
-        return 0  # #1801.
     #@+node:ekr.20110605121601.18123: *4* qlew.hasSelection
     def hasSelection(self) -> bool:
         """QHeadlineWrapper."""
@@ -424,13 +424,10 @@ class QLineEditWrapper(QTextMixin):
         w = self.widget
         if s is None:
             s = w.text()
-        i = self.toPythonIndex(i)
         i = max(0, min(i, len(s)))
         w.setCursorPosition(i)
     #@+node:ekr.20110605121601.18130: *4* qlew.setSelectionRange
-    def setSelectionRange(self,
-        i: Index, j: Index, insert: Index=None, s: str=None,
-    ) -> None:
+    def setSelectionRange(self, i: int, j: int, insert: Optional[int]=None, s: str=None) -> None:
         """QHeadlineWrapper."""
         if not self.check():
             return
@@ -440,14 +437,11 @@ class QLineEditWrapper(QTextMixin):
         if s is None:
             s = w.text()
         n = len(s)
-        i = self.toPythonIndex(i)
-        j = self.toPythonIndex(j)
         i = max(0, min(i, n))
         j = max(0, min(j, n))
         if insert is None:
             insert = j
         else:
-            insert = self.toPythonIndex(insert)
             insert = max(0, min(insert, n))
         if i == j:
             w.setCursorPosition(i)
@@ -458,7 +452,6 @@ class QLineEditWrapper(QTextMixin):
                 w.setSelection(j, -length)
             else:
                 w.setSelection(i, length)
-    # setSelectionRangeHelper = setSelectionRange
     #@-others
 #@+node:ekr.20150403094619.1: ** class LeoLineTextWidget(QFrame)
 class LeoLineTextWidget(QtWidgets.QFrame):  # type:ignore
@@ -506,7 +499,7 @@ if QtWidgets:
         """A subclass of QTextBrowser that overrides the mouse event handlers."""
         #@+others
         #@+node:ekr.20110605121601.18006: *3*  lqtb.ctor
-        def __init__(self, parent: Widget, c: Cmdr, wrapper: Wrapper) -> None:
+        def __init__(self, parent: Widget, c: Cmdr, wrapper: Any) -> None:  # wrapper is a LeoQtBody.
             """ctor for LeoQTextBrowser class."""
             for attr in ('leo_c', 'leo_wrapper',):
                 assert not hasattr(QtWidgets.QTextBrowser, attr), attr
@@ -1105,7 +1098,7 @@ class NumberBar(QtWidgets.QFrame):  # type:ignore
         # The y offset of the first line of the gutter.
         self.y_adjust = c.config.getInt('gutter-y-adjust') or 10
     #@+node:ekr.20181005085507.1: *3* NumberBar.mousePressEvent
-    def mousePressEvent(self, event: Event) -> None:
+    def mousePressEvent(self, event: MousePressEvent) -> None:
 
         c = self.c
 
@@ -1141,7 +1134,7 @@ class NumberBar(QtWidgets.QFrame):  # type:ignore
         # Always allocate room for 2 columns
         #width = self.fm.width(str(max(1000, self.highest_line))) + self.w_adjust
         if isQt6:
-            width = self.fm.boundingRect(str(max(1000, self.highest_line))).width()
+            width = self.fm.boundingRect(str(max(1000, self.highest_line))).width() + self.w_adjust
         else:
             width = self.fm.width(str(max(1000, self.highest_line))) + self.w_adjust
         if self.width() != width:
@@ -1305,7 +1298,7 @@ class QMinibufferWrapper(QLineEditWrapper):
         # level sheet will get pushed down quite frequently.
         self.widget.setStyleSheet(self.c.frame.top.styleSheet())
 
-    def setSelectionRange(self, i: Index, j: Index, insert: Index=None, s: str=None) -> None:
+    def setSelectionRange(self, i: int, j: int, insert: int=None, s: str=None) -> None:
         QLineEditWrapper.setSelectionRange(self, i, j, insert, s)
         insert = j if insert is None else insert
         if self.widget:
@@ -1354,13 +1347,11 @@ class QScintillaWrapper(QTextMixin):
         w.setAutoIndent(True)
     #@+node:ekr.20110605121601.18107: *3* qsciw.WidgetAPI
     #@+node:ekr.20140901062324.18593: *4* qsciw.delete
-    def delete(self, i: Index, j: Index=None) -> None:
+    def delete(self, i: int, j: int=None) -> None:
         """Delete s[i:j]"""
         w = self.widget
-        i = self.toPythonIndex(i)
         if j is None:
             j = i + 1
-        j = self.toPythonIndex(j)
         self.setSelectionRange(i, j)
         try:
             self.changingText = True  # Disable onTextChanged
@@ -1413,7 +1404,7 @@ class QScintillaWrapper(QTextMixin):
             # w = self.widget # A QsciScintilla widget.
             self.flashCount = flashes
             self.flashIndex1 = self.getInsertPoint()
-            self.flashIndex = self.toPythonIndex(i)
+            self.flashIndex = i
             self.flashBg = None if bg.lower() == 'same' else bg
             self.flashFg = None if fg.lower() == 'same' else fg
             addFlashCallback()
@@ -1423,8 +1414,6 @@ class QScintillaWrapper(QTextMixin):
         # https://bugs.launchpad.net/leo-editor/+bug/979142
         # https://bugs.launchpad.net/leo-editor/+bug/971166
         s = self.getAllText()
-        i = self.toPythonIndex(i)
-        j = self.toPythonIndex(j)
         return s[i:j]
     #@+node:ekr.20110605121601.18108: *4* qsciw.getAllText
     def getAllText(self) -> str:
@@ -1459,10 +1448,9 @@ class QScintillaWrapper(QTextMixin):
         """Return True if a QsciScintilla widget has a selection range."""
         return self.widget.hasSelectedText()
     #@+node:ekr.20140901062324.18601: *4* qsciw.insert
-    def insert(self, i: Index, s: str) -> int:
+    def insert(self, i: int, s: str) -> int:
         """Insert s at position i."""
         w = self.widget
-        i = self.toPythonIndex(i)
         w.SendScintilla(w.SCI_SETSEL, i, i)
         w.SendScintilla(w.SCI_ADDTEXT, len(s), g.toEncodedString(s))
         i += len(s)
@@ -1513,7 +1501,6 @@ class QScintillaWrapper(QTextMixin):
         # Ok for now.  Using SCI_SETYCARETPOLICY might be better.
         w = self.widget
         s = self.getAllText()
-        i = self.toPythonIndex(i)
         row, col = g.convertPythonIndexToRowCol(s, i)
         w.ensureLineVisible(row)
     #@+node:ekr.20110605121601.18113: *4* qsciw.setAllText
@@ -1527,17 +1514,15 @@ class QScintillaWrapper(QTextMixin):
     def setInsertPoint(self, i: int, s: str=None) -> None:
         """Set the insertion point in a QsciScintilla widget."""
         w = self.widget
-        i = self.toPythonIndex(i)
         # w.SendScintilla(w.SCI_SETCURRENTPOS,i)
         # w.SendScintilla(w.SCI_SETANCHOR,i)
         w.SendScintilla(w.SCI_SETSEL, i, i)
     #@+node:ekr.20110605121601.18115: *4* qsciw.setSelectionRange
-    def setSelectionRange(self, i: Index, j: int, insert: Index=None, s: str=None) -> None:
+    def setSelectionRange(self, i: int, j: int, insert: int=None, s: str=None) -> None:
         """Set the selection range in a QsciScintilla widget."""
         w = self.widget
-        i = self.toPythonIndex(i)
-        j = self.toPythonIndex(j)
-        insert = j if insert is None else self.toPythonIndex(insert)
+        if insert is None:
+            insert = j
         if insert >= i:
             w.SendScintilla(w.SCI_SETSEL, i, j)
         else:
@@ -1624,13 +1609,11 @@ class QTextEditWrapper(QTextMixin):
     #@+node:ekr.20110605121601.18078: *3* qtew.High-level interface
     # These are all widget-dependent
     #@+node:ekr.20110605121601.18079: *4* qtew.delete (avoid call to setAllText)
-    def delete(self, i: Index, j: Index=None) -> None:
+    def delete(self, i: int, j: int=None) -> None:
         """QTextEditWrapper."""
         w = self.widget
-        i = self.toPythonIndex(i)
         if j is None:
             j = i + 1
-        j = self.toPythonIndex(j)
         if i > j:
             i, j = j, i
         sb = w.verticalScrollBar()
@@ -1743,10 +1726,9 @@ class QTextEditWrapper(QTextMixin):
         """QTextEditWrapper."""
         return self.widget.textCursor().hasSelection()
     #@+node:ekr.20110605121601.18089: *4* qtew.insert (avoid call to setAllText)
-    def insert(self, i: Index, s: str) -> None:
+    def insert(self, i: int, s: str) -> None:
         """QTextEditWrapper."""
         w = self.widget
-        i = self.toPythonIndex(i)
         cursor = w.textCursor()
         try:
             self.changingText = True  # Disable onTextChanged.
@@ -1914,18 +1896,25 @@ class QTextEditWrapper(QTextMixin):
         finally:
             self.changingText = False
     #@+node:ekr.20110605121601.18095: *4* qtew.setInsertPoint
-    def setInsertPoint(self, i: Index, s: str=None) -> None:
+    def setInsertPoint(self, i: int, s: str=None) -> None:
         # Fix bug 981849: incorrect body content shown.
         # Use the more careful code in setSelectionRange.
         self.setSelectionRange(i=i, j=i, insert=i, s=s)
     #@+node:ekr.20110605121601.18096: *4* qtew.setSelectionRange
-    def setSelectionRange(self, i: Index, j: int, insert: Index=None, s: str=None) -> None:
+    def setSelectionRange(self,
+        i: Optional[int],
+        j: Optional[int],
+        insert: Optional[int]=None,
+        s: Optional[str]=None,
+    ) -> None:
         """Set the selection range and the insert point."""
         #
         # Part 1
         w = self.widget
-        i = self.toPythonIndex(i)
-        j = self.toPythonIndex(j)
+        if i is None:
+            i = 0
+        if j is None:
+            j = i
         if s is None:
             s = self.getAllText()
         n = len(s)
@@ -1934,7 +1923,7 @@ class QTextEditWrapper(QTextMixin):
         if insert is None:
             ins = max(i, j)
         else:
-            ins = self.toPythonIndex(insert)
+            ins = j if insert is None else insert
             ins = max(0, min(ins, n))
         #
         # Part 2:
@@ -1971,55 +1960,28 @@ class QTextEditWrapper(QTextMixin):
         v.selectionLength = j - i
         v.scrollBarSpot = w.verticalScrollBar().value()
     #@+node:ekr.20141103061944.40: *4* qtew.setXScrollPosition
-    def setXScrollPosition(self, pos: Index) -> None:
+    def setXScrollPosition(self, pos: int) -> None:
         """Set the position of the horizonatl scrollbar."""
         if pos is not None:
             w = self.widget
             sb = w.horizontalScrollBar()
             sb.setSliderPosition(pos)
     #@+node:ekr.20110605121601.18098: *4* qtew.setYScrollPosition
-    def setYScrollPosition(self, pos: Index) -> None:
+    def setYScrollPosition(self, pos: int) -> None:
         """Set the vertical scrollbar position."""
         if pos is not None:
             w = self.widget
             sb = w.verticalScrollBar()
             sb.setSliderPosition(pos)
-    #@+node:ekr.20110605121601.18100: *4* qtew.toPythonIndex
-    def toPythonIndex(self, index: Index, s: str=None) -> int:
-        """This is much faster than versions using g.toPythonIndex."""
-        w = self
-        te = self.widget
-        if index is None:
-            return 0
-        if isinstance(index, int):
-            return index
-        if index == '1.0':
-            return 0
-        if index == 'end':
-            return w.getLastPosition()
-        doc = te.document()
-        data = index.split('.')
-        if len(data) == 2:
-            row1, col1 = data
-            row, col = int(row1), int(col1)
-            bl = doc.findBlockByNumber(row - 1)
-            return bl.position() + col
-        g.trace(f"bad string index: {index}")
-        return 0
-    #@+node:ekr.20110605121601.18101: *4* qtew.toPythonIndexRowCol
-    def toPythonIndexRowCol(self, index: int) -> Tuple[int, int, int]:
-        w = self
-        if index == '1.0':
-            return 0, 0, 0
-        if index == 'end':
-            index = w.getLastPosition()
+    #@+node:ekr.20110605121601.18101: *4* qtew.toPythonIndexRowCol (fast)
+    def toPythonIndexRowCol(self, i: int) -> Tuple[int, int]:
+
         te = self.widget
         doc = te.document()
-        i = w.toPythonIndex(index)
         bl = doc.findBlock(i)
         row = bl.blockNumber()
         col = i - bl.position()
-        return i, row, col
+        return row, col
     #@-others
 #@-others
 
